import * as React from 'react';
import '../../../css/styles.css';
import { ChatListing } from '../ChatListing';
import { MessageView } from '../MessageView';

class ChatView extends React.Component<IChatView, {}> {
  constructor(props: IChatView) {
    super(props);
  }

  public onChatSelect(chatToSelect: IChatModel, name: string): void {
    this.props.onStateChange(name);
  }

  public render() {
    let component = null;
    const chatId = this.props.chatId;

    if (chatId) {
      const selectedChat = this.props.chatCollection.filter((chat) => { return chat.name === chatId; });
      const listItems = selectedChat[0].messages.map((item, index) => {
        return (
          <MessageView key={index} author={item.author} message={item.message} time={item.time}
                       msgId={item.msgId} tags={item.tags} contextString={item.contextString}
          fileName={item.fileName} filePath={item.filePath} fileContentType={item.fileContentType} />
        );
      });

      component = (
          <ul className="scrollbar">
            {listItems}
          </ul>
      );
    } else {
      const listItems = (
          this.props.chatCollection.map((chat, index) => {
            return (
                <ChatListing key={index}
                             name={chat.name}
                             participants={chat.participants.length}
                             onSelect={this.onChatSelect.bind(this, chat, chat.name)} />
            );
          })
      );

      component = (
          <ul className="chat-collection">
            {listItems}
          </ul>
      );
    }
    return (
        <div>
          {component}
        </div>
    );
  }
}

export { ChatView };
