import * as React from 'react';
import * as ReactDOM from 'react-dom';
import {ChatView} from '../ChatView';
import {ENTER_KEY, DATE_FORMAT} from '../../model/constants';
import * as moment from 'moment';
import '../../../css/styles.css';
import '../../../css/fdstyles.css';
// declare var emojify;
let flowdockConfig = require('../../config/flowdockconfig.ts');

class ChatWindow extends React.Component<IAppProps, IAppState> {

    public state: IAppState;

    constructor(props: IAppProps) {
        super(props);
        this.state = {chatId: null, messages: 0};
    }

    public handleNewMessageKeyDown(event: __React.KeyboardEvent): void {
        if (event.keyCode !== ENTER_KEY) {
            return;
        }
        event.preventDefault();

        this.addMessage();
    }

    public getChatModel(name: string): IChatModel {
        return this.props.chatCollection.filter((chat) => {
            return chat.name === name;
        })[0];
    }

    public onStateChange(name: string): void {
        if (name) {
            this.state.chatId = name;
            this.state.messages = this.getChatModel(name).messages.length;
            this.setState(this.state);

            // setTimeout(function () {
            //     window.scrollTo(0, document.body.scrollHeight);
            //     emojify.run();
            // });
        } else {
            this.state.chatId = null;
            this.state.messages = 0;
            this.setState(this.state);
        }
    }

    public addMessage(): void {
        const chat = this.getChatModel(this.state.chatId);
        /* tslint:disable:no-string-literal */
        const input = ReactDOM.findDOMNode<HTMLInputElement>(this.refs['newMessage']);
        /* tslint:enable:no-string-literal */
        let val = input.value.trim();

        let mydata = {
            content: val,
            event: flowdockConfig.eventName,
            tags: chat.productName + '_' + chat.portfolioName   };
        fetch(flowdockConfig.flowUrl + '/' + flowdockConfig.orgName + '/' +
            flowdockConfig.flowName + '/messages', {
            body: JSON.stringify(mydata),
            headers: flowdockConfig.apiHeaders,
            method: 'POST'      });
        if (val) {
            chat.addMessage(val, flowdockConfig.currentUser, moment().format(DATE_FORMAT));
            input.value = '';

            this.state.messages++;
            this.setState(this.state);

            // setTimeout(function () {
            //     window.scrollTo(0, document.body.scrollHeight);
            //     emojify.run();
            // });
        }
    }

    public render() {
        let header;
        // let footer;
        let fdFooter;

        let chatView = (
            <ChatView chatCollection={this.props.chatCollection}
                      chatId={this.state.chatId}
                      onStateChange={this.onStateChange.bind(this)}/>
        );

        if (this.state.chatId) {
            const chat = this.getChatModel(this.state.chatId);
            header = (
                <header className="chat-header">
          <span className="back-button">
            <button type="button" className="btn btn-primary"
                    onClick={this.onStateChange.bind(this , null)}>Back</button>
          </span>
                    <span className="chat-title">{this.state.chatId}</span>
          <span className="participants" title={chat.participants.join(', ')}>
            {chat.participants.length} <i className="glyphicon glyphicon-user"></i>
          </span>
                </header>
            );
        } else {
            header = (
                <header>
                    <h1>Sample Collaboration widget</h1>
                </header>
            );
        }

        if (this.state.chatId) {
            fdFooter = (
                <form id="chat-form" className="message-form">
                    <div className="textarea-autocompleter-area"></div>
                    <fieldset>
                        <ol>
                            <li className="left">
                            </li>
                            <li className="send right">
                                <a className="input-button send-button" type="submit">Send</a>
                                <a className="input-button tipsy-tooltip upload-button enabled"
                                   data-tipsy-gravity="s">
                                    <input id="file-input" type="file"/>
                                </a>
                            </li>
                            <li className="textarea no-overflow">
                                <div className="expanding-input">
                                    <pre><span className="copy"></span><br/></pre>
                                    <textarea className="message-input"
                                              dir="auto" maxLength="8000"
                                              placeholder="Type to start a new thread in the PPM flow…"
                                              ref="newMessage"
                                              onKeyDown={ e => this.handleNewMessageKeyDown(e) }>
                                    </textarea>
                                </div>
                                <a className="emoji-picker-toggle"><i className="fa fa-smile-o"></i></a>
                            </li>
                        </ol>
                    </fieldset>
                </form>
            );
        }

        return (
            <div className="main">
                {header}
                {chatView}
                {fdFooter}
            </div>
        );
    }
}

export {ChatWindow}
