import * as React from 'react';
import '../../../css/styles.css';
import '../../../css/fdstyles.css';
import {ENTER_KEY} from '../../model/constants';
import {TagView} from '../TagView';
let flowdockConfig = require('../../config/flowdockconfig.ts');
class MessageView extends React.Component<IMessageModel, ITagProps> {

    constructor(props: IMessageModel) {
        super(props);
        this.state = {
            childVisible: false, msgId: 0, showThread: false, tags: [] };
    }

    public toggleTagView(): void {
        this.setState({childVisible: !this.state.childVisible, msgId: 0,
            showThread: false, tags: [] });
    }

    public renderMsg(message: string, fileProps: any): any {
        if (fileProps.fileName === '') {
            if (message.indexOf('#') !== -1) {
                let re = /#[a-zA-Z0-9_\d]+/ig;
                let tags = message.match(re);
                tags.map(function (item) {
                    message = message.replace(item, '<span style="color:darkblue">' + item + '</span>');
                });
            }

            if (message.indexOf('@') !== -1) {
                let re = /@[a-zA-Z0-9_\d]+/ig;
                let tags = message.match(re);
                tags.map(function (item) {
                    message = message.replace(item, '<span style="color:orange">' + item + '</span>');
                });
            }
            return message;
        } else {
            let url = flowdockConfig.attachmentUrl + fileProps.filePath;
            return (
                    <a href={url}>
                        {fileProps.fileName}
                    </a>
                );
        }
    }

    public toggleReply() {
        this.setState({childVisible: false, msgId: 0,
            showThread: !this.state.showThread, tags: [] });
    }

    public postThreadMsg(event: __React.KeyboardEvent): void {
        if (event.keyCode !== ENTER_KEY) {
            return;
        }
        console.log( 'Threaded msg entered..' );
        event.preventDefault();
        // write post thread msg here
    }

    public renderReply() {
        if (this.state.showThread === true) {
            return (
                <div id="thread-msg" className="thread-input n">
                    <ul className="autocompleter hidden">
                    </ul>
                    <div className="tokenist focused" contentEditable="plaintext-only"
                         spellCheck="false" onKeyDown={e => this.postThreadMsg(e)}
                    >
                        <div id="thread-text">
                        </div>
                        <span className="tokenist-space">
                        </span></div>
                </div>
            );
        } else {
            return ( <div></div> );
        }
    }

    public renderTagView() {
        return (
            <TagView childVisible = {this.state.childVisible} tags={this.props.tags}
                     msgId={this.props.msgId} showThread={this.state.showThread}/>
        );
    }

    public modifyTags(contextString: string) {

      let filteredtags = this.props.tags.filter (
        function(e){
          return e !== contextString;
        });
      let hashtags = '';
      filteredtags.map((item, index) => {
        if (this.props.message.indexOf(item) === -1) {
          if (item.indexOf('@') === -1) {
            item = '  #' + item;
          } else {
            item = '  ' + item;
          }
          if (item.indexOf('@') !== -1) {
            item = item.replace(item, '<span style="color:orange">' + item + '</span>');
          }
          if (item.indexOf('#') !== -1) {
            item = item.replace(item, '<span style="color:darkblue">' + item + '</span>');
          }
          hashtags = hashtags + item;
        }
      });
      return hashtags;
    }
    public render() {
        let menu = (
            <menu className="message-actions clean">
                <li>
                    <a className="no-follow-link bubble tipsy-tooltip thread green"
                       title="Reply from thread" onClick={this.toggleReply.bind(this)}>
      <span className="fa-stack">
        <i className="fa fa-circle fa-stack-2x"></i>
        <i className="fa fa-mail-reply fa-stack-1x fa-inverse"></i>
      </span>
                    </a>
                </li>
                <li>
                    <a className="edit-message" title="Edit this message.">
      <span className="fa-stack">
        <i className="fa fa-circle fa-stack-2x"></i>
        <i className="fa fa-pencil fa-stack-1x fa-inverse"></i>
      </span>
                    </a>
                </li>
                <li>
                    <a className="edit-tags filled" title="Edit message tags."
                       onClick={this.toggleTagView.bind(this)}>
      <span className="fa-stack">
        <i className="fa fa-circle fa-stack-2x"></i>
        <i className="fa fa-tag fa-stack-1x fa-inverse"></i>
      </span>
                    </a>
                </li>
            </menu>
        );
        let message = this.props.message.replace('#' + this.props.contextString, '');
        message = message.replace(this.props.contextString, '');
        let fileProps = {
            fileContType: this.props.fileContentType,
            fileName: this.props.fileName, filePath: this.props.filePath     };

        let msg = this.renderMsg(message, fileProps);
        function markdown(input) {
          return {
            __html : input,
          };
        }
        if (this.props.filePath === '') {
            return (
                <li className="chat-message message">
                    {menu}
                    <div className="message-author">{this.props.author}</div>
                    <div className="content" dir="auto">
                        <div className="msg-body" dangerouslySetInnerHTML={markdown(msg)}></div>
                        <footer className="msg-footer">
                            <div className="tags" dangerouslySetInnerHTML=
                                {markdown(this.modifyTags(this.props.contextString))}>
                            </div>
                        </footer>
                    </div>
                    {this.renderTagView()}
                    {this.renderReply()}
                </li>
            );
        } else {
            return (
                <li className="chat-message message">
                    {menu}
                    <div className="message-author">{this.props.author}</div>
                    <div className="content" dir="auto">
                        <div className="msg-body">{msg}</div>
                        <footer className="msg-footer">
                            <div className="tags" dangerouslySetInnerHTML=
                                {markdown(this.modifyTags(this.props.contextString))}>
                            </div>
                        </footer>
                    </div>
                    {this.renderTagView()}
                    {this.renderReply()}
                </li>
            );
        }
    }
}

export {MessageView};
